/*
 * Copyright (C) 2013-2014  Charles Winterhalter
 * School of Computing Science, Newcastle University
 *
 * This file is part of the Java Enrichment of Pathways Extended To Topology
 * (JEPETTO) plugin for Cytoscape.
 *
 * JEPETTO is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * JEPETTO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details:
 * http://www.gnu.org/licenses/gpl.html
 */

package org.cytoscape.jepetto.internal;

import java.awt.BorderLayout;
import java.awt.Desktop;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.net.URISyntaxException;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

public class JEPETTOAboutDialog extends JDialog {

	/*******************
	 *    VARIABLES    *
	 *******************/

	// Serial version UID (default)
	private static final long serialVersionUID = 1L;

	// Generated frame
	final public JFrame optionsFrame;



	/*******************
	 *   CONSTRUCTOR   *
	 *******************/

	/**
	 * Default constructor
	 */
	public JEPETTOAboutDialog () {
		optionsFrame = new JFrame("About JEPETTO");
		optionsFrame.setResizable(false);
		optionsFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		optionsFrame.setAlwaysOnTop(true);
		optionsFrame.setLocationRelativeTo(null);

		JEditorPane aboutPanel = createEditorPane();
		JPanel buttonPanel = createButton();

		optionsFrame.getContentPane().add(aboutPanel, BorderLayout.CENTER);
		optionsFrame.getContentPane().add(buttonPanel, BorderLayout.SOUTH);

		optionsFrame.pack();
		optionsFrame.setVisible(true);
	}



	/*******************
	 *     METHODS     *
	 *******************/

	/**
	 * Create a close button
	 * @return button panel
	 */
	private JPanel createButton(){
		JPanel buttonPanel = new JPanel();
		JButton button = new JButton("Close");
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				optionsFrame.dispose();
			}
		});
		button.setAlignmentX(CENTER_ALIGNMENT);
		buttonPanel.add(button);
		return buttonPanel;
	}

	/**
	 * Create the display panel "ABOUT"
	 * @return display panel
	 */
	private JEditorPane createEditorPane() {
		JEditorPane editorPane = new JEditorPane();
		editorPane.setMargin(new Insets(10, 10, 10, 10));
		editorPane.setEditable(false);
		editorPane.setOpaque(false);
		editorPane.setEditorKit(JEditorPane.createEditorKitForContentType("text/html"));

		String iconPath = getClass().getClassLoader().getResource("/logo.png").toString();
		String text = "<html><p align='center'><img src='" + iconPath + "' /><br /><br />"
			+ "<b>Java Enrichment of Pathways Extended To Topology</b><br>"
			+ "<small>version 1.3.1</small><br /><br />"
			+ "<b>Maintainer and main developer</b><br />"
			+ "Charles Winterhalter (Newcastle University)"
			+ "<br /><br /><b>Contributors</b><br />"
			+ "Paweł Widera, Natalio Krasnogor (Newcastle University)"
			+ "<br /><br /><b>Contact</b><br />"
			+ "<a href=\"mailto:jepetto.plugin@gmail.com\">jepetto.plugin@gmail.com</a>"
			+ "<br /><br /><b>References</b><br />"
			+ "<ul style='list-style-type: none;  margin: 0'>"
			+ "<li><a href=\"http://apps.cytoscape.org/apps/jepetto\">http://apps.cytoscape.org/apps/jepetto</a> (doi:10.1093/bioinformatics/btt732)</li>"
			+ "<li><a href=\"http://www.enrichnet.org/\">http://www.enrichnet.org/</a> (doi:10.1093/bioinformatics/bts389)</li>"
			+ "<li><a href=\"http://www.pathexpand.org/\">http://www.pathexpand.org/</a> (doi:10.1186/1471-2105-11-597)</li>"
			+ "<li><a href=\"http://www.topogsa.org/\">http://www.topogsa.org/</a> (doi:10.1093/bioinformatics/btq131)</li>"
			+ "</ul></p></html>";
		editorPane.setText(text);

		editorPane.addHyperlinkListener(new HyperlinkListener() {
			public void hyperlinkUpdate(HyperlinkEvent e) {
				if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
					if (Desktop.isDesktopSupported()) {
						try {
							Desktop.getDesktop().browse(e.getURL().toURI());
						} catch (IOException e1) {
							e1.printStackTrace();
						} catch (URISyntaxException e1) {
							e1.printStackTrace();
						}
					}
				}
			}
		});
		return editorPane;
	}
}
