/*
 * Copyright (C) 2013-2014  Charles Winterhalter
 * School of Computing Science, Newcastle University
 *
 * This file is part of the Java Enrichment of Pathways Extended To Topology
 * (JEPETTO) plugin for Cytoscape.
 *
 * JEPETTO is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * JEPETTO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details:
 * http://www.gnu.org/licenses/gpl.html
 */

package org.cytoscape.jepetto.internal;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;

public class JEPETTOEnrichNetResultPanel extends JPanel {

	/*******************
	 *    ATTRIBUTES   *
	 *******************/

	// Serial version ID (auto-generated)
	private static final long serialVersionUID = -6568530389292248147L;

	// Constructor attributes
	private final JTabbedPane resultsPanel;
	// UI components
	private final JPanel rankPanel = new JPanel();
	private final JPanel regressionPanel = new JPanel();
	private final JPanel mappingPanel = new JPanel();
	private final JLabel mappingLabel = new JLabel();
	private static JPanel tissueSpecPanel = new JPanel();
	private final JComboBox cytoscapeNetworkBox = new JComboBox();
	private final JTable rankingTable;
	protected String tableToSave = "Ranking";
	protected JRadioButton tissueTable = new JRadioButton("Tissue");




	/*******************
	 *   CONSTRUCTOR   *
	 *******************/

	/**
	 * Default constructor: no tissue-specificity
	 * @param rankTable
	 * @param mappingTable
	 * @param regressionPlotPanel
	 * @param regressionValues
	 * @param id
	 * @param database
	 * @param pathwaysList
	 */
	public JEPETTOEnrichNetResultPanel(JTable rankTable, JTable mappingTable,
		JPanel regressionPlotPanel, Double[] regressionValues, String id, String database,
		HashMap<String, String> pathwaysList, HashMap<String, Integer> networkComputationIndexesMap,
		String experimentTraceTooltip) {

		JPanel main = new JPanel(new BorderLayout());
		CyActivator.enrichmentResultsTab.getResultsTabbedPanel().addTab(CyActivator.overallEnrichmentAnalysisNumber+". Enrichment", main);
		JPanel pnlTab = new JPanel(new GridBagLayout());
		pnlTab.setOpaque(false);
		JLabel lblTitle = new JLabel(CyActivator.overallEnrichmentAnalysisNumber+". Enrichment");
		lblTitle.setToolTipText(experimentTraceTooltip);
		JButton btnClose = new JButton("x");
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.weightx = 1;
		pnlTab.add(lblTitle, gbc);
		gbc.gridx++;
		gbc.weightx = 0;
		pnlTab.add(btnClose, gbc);
		CyActivator.enrichmentResultsTab.getResultsTabbedPanel().setTabComponentAt(CyActivator.enrichmentResultsTab.getResultsTabbedPanel().getTabCount()-1, pnlTab);
		btnClose.addActionListener(new JEPETTOCloseTabHandling(CyActivator.overallEnrichmentAnalysisNumber+". Enrichment", CyActivator.enrichmentResultsTab));

		resultsPanel = new JTabbedPane();
		resultsPanel.setTabPlacement(JTabbedPane.LEFT);
		main.add(resultsPanel, BorderLayout.CENTER);

		this.rankingTable = rankTable;

		if (rankTable != null) {
			updateNetworkBox(rankTable);
			displayGeneralLayout();
			displayRegression(regressionPlotPanel, regressionValues);
			displayTable(rankTable, rankPanel);
			displayTable(mappingTable, mappingPanel);
			updateMappingLabelToMappedGenesNb(mappingTable);

			CyActivator.enrichmentResultsTab.getResultsTabbedPanel().setSelectedIndex(CyActivator.enrichmentResultsTab.getResultsTabbedPanel().getTabCount()-1);
		} else {
			JOptionPane.showMessageDialog(this.resultsPanel, "Unfortunately the analysis did not work. Please try with a bigger gene set.", "Gene set error", JOptionPane.ERROR_MESSAGE);
		}
	}


	/**
	 * Alternative constructor: tissue-specificity activated
	 * @param rankTable
	 * @param mappingTable
	 * @param tissueSpecTable
	 * @param regressionPlotPanel
	 * @param regressionValues
	 * @param id
	 * @param database
	 * @param pathwaysList
	 */
	public JEPETTOEnrichNetResultPanel(JTable rankTable,
			JTable mappingTable,
			JTable tissueSpecTable,
			JPanel regressionPlotPanel,
			Double[] regressionValues,
			String id,
			String database,
			HashMap<String, String> pathwaysList,
			HashMap<String, Integer> networkComputationIndexesMap,
			String experimentTraceTooltip) {

		this(rankTable, mappingTable, regressionPlotPanel, regressionValues, id,
			database, pathwaysList, networkComputationIndexesMap, experimentTraceTooltip);

		if (rankTable != null) {
			displayTissueSpecificityTab(tissueSpecTable);
		}
	}


	/*******************
	 *     METHODS     *
	 *******************/

	/**
	 * Update mapping table title to
	 * the number of mapped genes in the
	 * background interaction network
	 * @param mappingTable: table of input gene set mapping values on the background interaction network
	 */
	private void updateMappingLabelToMappedGenesNb(JTable mappingTable) {
		int presenceIndex = mappingTable.getColumn("Presence").getModelIndex();
		int realIndex = mappingTable.convertColumnIndexToModel(presenceIndex);
		int mappedGenesNumber = 0;
		for (int row = 0 ; row < mappingTable.getRowCount() ; row++)
			if (mappingTable.getValueAt(row, realIndex).equals("yes")) mappedGenesNumber++;
		mappingLabel.setText("<html><b>" + mappedGenesNumber + " out of " + mappingTable.getRowCount() + " genes mapped onto the interaction network:</b></html>");
		mappingPanel.validate();
		mappingPanel.repaint();
	}

	/**
	 * Update box
	 * @param table
	 */
	private void updateNetworkBox(JTable table) {
		cytoscapeNetworkBox.removeAll();
		int row = 1;
		while (row < table.getRowCount()) {
			cytoscapeNetworkBox.addItem(table.getModel().getValueAt(row, 0));
			row++;
		}
	}

	/**
	 * General structure
	 */
	private void displayGeneralLayout() {
		// Ranking panel
		JPanel rankingTabPanel = new JPanel();
		JScrollPane rankScoll = new JScrollPane(rankingTabPanel);
		resultsPanel.add("Ranking", rankScoll);
		displayRankingPanel(rankingTabPanel);
		// Mapping panel
		JPanel mappingDataPanel = new JPanel();
		JScrollPane mappingScroll = new JScrollPane(mappingDataPanel);
		resultsPanel.add("Mapping", mappingScroll);
		displayMappingPanel(mappingDataPanel);
		// Regression panel
		JScrollPane regressionScroll = new JScrollPane(regressionPanel);
		resultsPanel.add("Regression", regressionScroll);
	}

	/**
	 * Tissue-specifity tab
	 * @param table
	 */
	private void displayTissueSpecificityTab(JTable table) {
		JPanel tissueSpecTabPanel = new JPanel();
		JPanel tissuePanel = new JPanel(new BorderLayout());
		JScrollPane tissueSpecScoll = new JScrollPane(tissueSpecTabPanel);
		resultsPanel.add("Tissue", tissueSpecScoll);
		tissueSpecTabPanel.setLayout(new BorderLayout(2,4));
		JLabel tissueSpecLabel = new JLabel("<html><b>Tissue specific XD-scores:</b></html>");
		tissuePanel.add(tissueSpecLabel, BorderLayout.CENTER);
		JButton saveTissue = new JButton("Save");
		tissuePanel.add(saveTissue, BorderLayout.EAST);
		saveTissue.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				new JEPETTOSaveTable("Tissue", tissueSpecPanel, null);
			}
		});
		tissueSpecTabPanel.add(tissuePanel, BorderLayout.NORTH);
		tissueSpecTabPanel.add(tissueSpecPanel, BorderLayout.CENTER);
		displayTable(table, tissueSpecPanel);
		tissueTable.setEnabled(true);
	}

	/**
	 * Ranking tab
	 * @param panel
	 */
	private void displayRankingPanel(JPanel panel) {
		panel.setLayout(new BorderLayout(2,4));
		int height = (int)(0.5 * Toolkit.getDefaultToolkit().getScreenSize().getHeight());
		panel.setPreferredSize(new Dimension(500, height));
		// Label for the ranking
		JPanel labelPanel = new JPanel(new BorderLayout());
		JLabel rankLabel = new JLabel("<html><b>Comparison to reference database:</b><br />(double-click to generate network)</html>");
		labelPanel.add(rankLabel, BorderLayout.CENTER);
		JButton saveRanking = new JButton("Save");
		labelPanel.add(saveRanking, BorderLayout.EAST);
		saveRanking.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				new JEPETTOSaveTable("Ranking", rankPanel, rankingTable);
			}
		});
		panel.add(labelPanel, BorderLayout.NORTH);
		// Rank table
		panel.add(rankPanel, BorderLayout.CENTER);
		// Label for significance
		JPanel visualGraphPanel = new JPanel();
		panel.add(visualGraphPanel, BorderLayout.SOUTH);
		visualGraphPanel.setLayout(new BorderLayout());
		JLabel significantLabel = new JLabel("<html><em>Note: statistically significant results are highlighted.</em></html>");
		visualGraphPanel.add(significantLabel, BorderLayout.NORTH);
	}

	/**
	 * Update any table with new values
	 * @param table
	 * @param panel
	 */
	protected static void displayTable(JTable table, JPanel panel) {
		JScrollPane scroll = (panel == tissueSpecPanel) ? new JScrollPane(table, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS) : new JScrollPane(table);
		panel.removeAll();
		panel.setLayout(new BorderLayout());
		panel.add(table.getTableHeader(), BorderLayout.PAGE_START);
		panel.add(scroll, BorderLayout.CENTER);
		panel.validate();
	}

	/**
	 * Mapping tab
	 * @param panel
	 */
	private void displayMappingPanel(JPanel panel) {
		panel.setLayout(new BorderLayout(2,4));

		int height = (int)(0.5 * Toolkit.getDefaultToolkit().getScreenSize().getHeight());
		panel.setPreferredSize(new Dimension(500, height));

		// Mapping data
		JPanel mapPanel = new JPanel(new BorderLayout());
		mapPanel.add(mappingLabel, BorderLayout.CENTER);
		JButton saveMapping = new JButton("Save");
		mapPanel.add(saveMapping, BorderLayout.EAST);
		saveMapping.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				new JEPETTOSaveTable("Mapping", mappingPanel, null);
			}
		});
		panel.add(mapPanel, BorderLayout.NORTH);
		panel.add(mappingPanel, BorderLayout.CENTER);	// updated component
	}

	/**
	 * Display regression
	 */
	private void displayRegression(JPanel plotPanel, Double[] values) {
		regressionPanel.removeAll();

		regressionPanel.setPreferredSize(new Dimension(500, 600));

		if (values == null) {
			JLabel errorLabel = new JLabel("Unfortunately, the regression did not work. A larger input gene set is needed.");
			regressionPanel.add(errorLabel);
			regressionPanel.validate();
			return;
		}

		// title
		JLabel labelPlot = new JLabel("<html><b>Regression plot:</b> XD-score vs significance of overlap (Fisher test, q-value)</html>");
		regressionPanel.add(labelPlot);

		// panel for the plot
		regressionPanel.add(plotPanel);

		// panel for regression values
		JPanel regressionValuesPanel = new JPanel();
		regressionValuesPanel.setLayout(new GridLayout(2,1));
		regressionPanel.add(regressionValuesPanel);

		String text = "<html>Absolute Pearson correlation (XD-scores vs. q-values): <b>"
			+ values[0].toString() +"</b></html>";
		JLabel pearsonLabel = new JLabel(text);
		regressionValuesPanel.add(pearsonLabel);

		text = "<html>XD-score significance threshold: <b>" + values[1].toString() +"</b></html>";
		JLabel significanceLabel = new JLabel(text);

		text = "<html>Regression fit equivalent to Fisher q-value = 0.05 + upper bound"
			+ "<br />of 95% confidence interval for linear fitting.</html>";
		significanceLabel.setToolTipText(text);
		regressionValuesPanel.add(significanceLabel);

		regressionPanel.validate();
	}

}
