/*
 * Copyright (C) 2013-2014  Charles Winterhalter
 * School of Computing Science, Newcastle University
 *
 * This file is part of the Java Enrichment of Pathways Extended To Topology
 * (JEPETTO) plugin for Cytoscape.
 *
 * JEPETTO is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * JEPETTO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details:
 * http://www.gnu.org/licenses/gpl.html
 */

package org.cytoscape.jepetto.internal;

import java.awt.Desktop;
import java.awt.event.ActionEvent;
import java.net.URL;

import javax.help.CSH.DisplayHelpFromSource;
import javax.help.HelpBroker;

import org.cytoscape.application.swing.AbstractCyAction;
import org.cytoscape.application.swing.CySwingApplication;
import org.cytoscape.application.swing.CytoPanel;
import org.cytoscape.application.swing.CytoPanelName;
import org.cytoscape.application.swing.CytoPanelState;

public class JEPETTOMenuActionPlugin extends AbstractCyAction {

	/*******************
	 *   ATTRIBUTES    *
	 *******************/

	// Serial version UID (auto-generated)
	private static final long serialVersionUID = 8292806967891823933L;

	// Cytoscape swing service
	private final CySwingApplication desktopApp;

	// Cytoscape panels
	private final CytoPanel cytoPanelWest;
	private final JEPETTOServerPanel myCytoPanel;

	// Server service
	private String serverName;



	/*******************
	 *   CONSTRUCTOR   *
	 *******************/

	/**
	 * Default constructor
	 * @param desktopApp
	 * @param myCytoPanel
	 * @param servname
	 */
	public JEPETTOMenuActionPlugin(CySwingApplication desktopApp,
			JEPETTOServerPanel myCytoPanel,
			String name){
		super(name);
		this.name = name;
		setPreferredMenu("Apps.JEPETTO");
		setMenuGravity(2.0f);
		this.desktopApp = desktopApp;
		this.cytoPanelWest = this.desktopApp.getCytoPanel(CytoPanelName.WEST);
		this.myCytoPanel = myCytoPanel;
	}



	/*******************
	 *     METHODS     *
	 *******************/

	/**
	 * Listener for menu subitems
	 * @param user action
	 */
	@Override
	public void actionPerformed(ActionEvent event) {
		if (name == "Topological analysis") {
			selectInputPanel();
			JEPETTOServerPanel.topologyRadio.doClick();
		} else if (name == "Enrichment analysis") {
			selectInputPanel();
			JEPETTOServerPanel.enrichmentRadio.doClick();
		} else if (name == "About") {
			new JEPETTOAboutDialog();
		} else if (name == "Online tutorial") {
			try {
				Desktop.getDesktop().browse(new URL("http://apps.cytoscape.org/apps/jepetto").toURI());
			} catch (Exception e) {
				e.printStackTrace();
			}
		} else if (name == "Help") {
			HelpBroker helpBroker = CyActivator.helpBroker.getHelpBroker();
			helpBroker.setCurrentID("index");
			DisplayHelpFromSource action = new DisplayHelpFromSource(helpBroker);
			action.actionPerformed(event);
		}
	}

	/**
	 * Select Cytoscape control panel
	 */
	public void selectInputPanel() {
		// If the state of the cytoPanelWest is HIDE, show it
		if (cytoPanelWest.getState() == CytoPanelState.HIDE) {
			cytoPanelWest.setState(CytoPanelState.DOCK);
		}
		// Select my panel
		int index = cytoPanelWest.indexOfComponent(myCytoPanel);
		if (index == -1) {
			return;
		}
		cytoPanelWest.setSelectedIndex(index);
	}
}

