/*
 * Copyright (C) 2013-2014  Charles Winterhalter
 * School of Computing Science, Newcastle University
 *
 * This file is part of the Java Enrichment of Pathways Extended To Topology
 * (JEPETTO) plugin for Cytoscape.
 *
 * JEPETTO is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * JEPETTO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details:
 * http://www.gnu.org/licenses/gpl.html
 */

package org.cytoscape.jepetto.internal;

import org.cytoscape.view.model.events.NetworkViewAboutToBeDestroyedEvent;
import org.cytoscape.view.model.events.NetworkViewAboutToBeDestroyedListener;
import org.cytoscape.view.model.events.NetworkViewAddedEvent;
import org.cytoscape.view.model.events.NetworkViewAddedListener;

public class JEPETTONetworkViewListener implements NetworkViewAddedListener, NetworkViewAboutToBeDestroyedListener  {

	/**
	 * Handles "use selected node" buttons ON state
	 * while adding a network view to cytoscape
	 */
	@Override
	public void handleEvent(NetworkViewAddedEvent e) {
		// Enable the "Use selected nodes" button in enrichment and topology input panels
		JEPETTOEnrichNetPanel.selectedNodesStartButton.setEnabled(true);
		JEPETTOTopoGSAPanel.selectedNodesStartButton.setEnabled(true);
	}

	/**
	 * Handles "use selected node" buttons OFF state
	 * while deleting a network view in Cytoscape
	 */
	@Override
	public void handleEvent(NetworkViewAboutToBeDestroyedEvent e) {
		// Check whether there are no more network views
		if (CyActivator.networkViewManager.getNetworkViewSet().size() == 0) {
			// Disable "Use selected nodes" button in enrichment and topology input panels
			JEPETTOEnrichNetPanel.selectedNodesStartButton.setEnabled(false);
			JEPETTOTopoGSAPanel.selectedNodesStartButton.setEnabled(false);
		}
	}

}
