/*
 * Copyright (C) 2013-2014  Charles Winterhalter
 * School of Computing Science, Newcastle University
 *
 * This file is part of the Java Enrichment of Pathways Extended To Topology
 * (JEPETTO) plugin for Cytoscape.
 *
 * JEPETTO is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * JEPETTO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details:
 * http://www.gnu.org/licenses/gpl.html
 */

package org.cytoscape.jepetto.internal;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Hashtable;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class JEPETTOOptionDialog extends JDialog {

	/*******************
	 *   ATTRIBUTES    *
	 *******************/

	// Serial version UID (default)
	private static final long serialVersionUID = 1L;



	/*******************
	 *   CONSTRUCTOR   *
	 *******************/

	/**
	 * Default constructor
	 */
	public JEPETTOOptionDialog () {
		JFrame optionsFrame = new JFrame("Advanced options (JEPETTO)");
		optionsFrame.setResizable(false);
		optionsFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		optionsFrame.setAlwaysOnTop(true);
		optionsFrame.setLocationRelativeTo(null);
		JPanel parametersPanel = createParamPanel();
		optionsFrame.getContentPane().add(parametersPanel, BorderLayout.CENTER);
		optionsFrame.pack();
		optionsFrame.setVisible(true);
	}



	/*******************
	 *     METHODS     *
	 *******************/

	/**
	 * Create parameters panel
	 * @return parameters panel
	 */
	private JPanel createParamPanel() {
		TitledBorder importNetworkFrame = BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.black), "Path expansion criteria");
		importNetworkFrame.setTitleJustification(TitledBorder.LEFT);
		JPanel panel = new JPanel(new BorderLayout());
		panel.setBorder(importNetworkFrame);
		JPanel optionsPan = new JPanel(new GridLayout(3,2));
		JPanel updatePan = new JPanel();
		final JButton applyModif = new JButton("Apply");
		final JButton defaultParam = new JButton("Default");
		applyModif.setEnabled(false);
		updatePan.add(defaultParam);
		updatePan.add(applyModif);
		panel.add(updatePan, BorderLayout.SOUTH);

		// Association threshold
		Hashtable<Integer, JLabel> associationValuesTable = new Hashtable<Integer, JLabel>();
		associationValuesTable.put(10, new JLabel("1.0"));
		associationValuesTable.put(12, new JLabel("1.2"));
		associationValuesTable.put(14, new JLabel("1.4"));
		associationValuesTable.put(16, new JLabel("1.6"));

		final JSlider associationSlider = new JSlider(JSlider.HORIZONTAL, 10, 16, (int) (CyActivator.associationValue*10));
		associationSlider.setLabelTable(associationValuesTable);
		associationSlider.setFont(new Font("Serif", Font.ITALIC, 7));
		associationSlider.setForeground(Color.black);
		associationSlider.setMajorTickSpacing(2);
		associationSlider.setMinorTickSpacing(1);
		associationSlider.setPaintTicks(true);
		associationSlider.setPaintLabels(true);

		final JLabel associationLabel = new JLabel("<html><em>Association threshold:</em><b> "+(associationSlider.getValue()/10.0)+"</b></html>");
		String text = "<html>A minimum ratio of interactions between the <b>candidate node</b><br />"
			+ "and the <b>pathway</b> vs interactions between the <b>candidate</b><br />"
			+ "and the <b>rest</b> of the network.</html>";
		associationLabel.setToolTipText(text);

		associationSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				associationLabel.setText("<html><em>Association threshold:</em><b> "+(associationSlider.getValue()/10.0)+"</b></html>");
				applyModif.setEnabled(true);
			}
		});

		optionsPan.add(associationLabel);
		optionsPan.add(associationSlider);

		// Coverage threshold
		Hashtable<Integer, JLabel> coverageValuesTable = new Hashtable<Integer, JLabel>();
		coverageValuesTable.put(1, new JLabel("0.1"));
		coverageValuesTable.put(3, new JLabel("0.3"));
		coverageValuesTable.put(5, new JLabel("0.5"));
		coverageValuesTable.put(7, new JLabel("0.7"));
		coverageValuesTable.put(9, new JLabel("0.9"));

		final JSlider coverageSlider = new JSlider(JSlider.HORIZONTAL, 1, 10, (int) (CyActivator.coverageValue*10));
		coverageSlider.setLabelTable(coverageValuesTable);
		coverageSlider.setFont(new Font("Serif", Font.ITALIC, 7));
		coverageSlider.setForeground(Color.black);
		coverageSlider.setMajorTickSpacing(2);
		coverageSlider.setMinorTickSpacing(1);
		coverageSlider.setPaintTicks(true);
		coverageSlider.setPaintLabels(true);

		final JLabel coverageLabel = new JLabel("<html><em>Coverage threshold:</em><b> "+(coverageSlider.getValue()/10.0)+"</b></html>");
		text = "<html>A minimum ratio of <b>pathway members</b><br />"
			+ "connected to the <b>candidate node</b>.</html>";
		coverageLabel.setToolTipText(text);

		coverageSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				coverageLabel.setText("<html><em>Coverage threshold:</em><b> "+(coverageSlider.getValue()/10.0)+"</b></html>");
				applyModif.setEnabled(true);
			}
		});

		optionsPan.add(coverageLabel);
		optionsPan.add(coverageSlider);

		// Triangles threshold
		Hashtable<Integer, JLabel> triangleValuesTable = new Hashtable<Integer, JLabel>();
		triangleValuesTable.put(1, new JLabel("0.1"));
		triangleValuesTable.put(3, new JLabel("0.3"));
		triangleValuesTable.put(5, new JLabel("0.5"));
		triangleValuesTable.put(7, new JLabel("0.7"));
		triangleValuesTable.put(9, new JLabel("0.9"));

		final JSlider triangleSlider = new JSlider(JSlider.HORIZONTAL, 1, 10, (int) (CyActivator.triangleValue*10));
		triangleSlider.setLabelTable(triangleValuesTable);
		triangleSlider.setFont(new Font("Serif", Font.ITALIC, 7));
		triangleSlider.setForeground(Color.black);
		triangleSlider.setMajorTickSpacing(2);
		triangleSlider.setMinorTickSpacing(1);
		triangleSlider.setPaintTicks(true);
		triangleSlider.setPaintLabels(true);

		final JLabel triangleLabel = new JLabel("<html><em>Triangle threshold:</em><b> "+(triangleSlider.getValue()/10.0)+"</b></html>");
		text = "<html>A minimum ratio of <b>triangles</b> between the <b>candidate node</b>,<br />"
			+ "a member of the <b>pathway</b> and <b>another node</b> connected to the pathway<br />"
			+ "vs all such possible triangles.</html>";
		triangleLabel.setToolTipText(text);

		triangleSlider.setToolTipText("Corresponds to topological triangles connecting a candidate to pathway nodes compared to the total number of possible triangles");
		triangleSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				triangleLabel.setText("<html><em>Triangle threshold:</em><b> "+(triangleSlider.getValue()/10.0)+"</b></html>");
				applyModif.setEnabled(true);
			}
		});

		optionsPan.add(triangleLabel);
		optionsPan.add(triangleSlider);

		panel.add(optionsPan, BorderLayout.CENTER);

		applyModif.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				CyActivator.associationValue = associationSlider.getValue()/10.0;
				CyActivator.coverageValue = coverageSlider.getValue()/10.0;
				CyActivator.triangleValue = triangleSlider.getValue()/10.0;
				applyModif.setEnabled(false);
			}
		});

		defaultParam.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				CyActivator.associationValue = 1.0;
				CyActivator.coverageValue = 0.3;
				CyActivator.triangleValue = 0.1;
				associationSlider.setValue((int) (CyActivator.associationValue*10));
				coverageSlider.setValue((int) (CyActivator.coverageValue*10));
				triangleSlider.setValue((int) (CyActivator.triangleValue*10));
			}
		});
		return panel;
	}
}
