/*
 * Copyright (C) 2013-2014  Charles Winterhalter
 * School of Computing Science, Newcastle University
 *
 * This file is part of the Java Enrichment of Pathways Extended To Topology
 * (JEPETTO) plugin for Cytoscape.
 *
 * JEPETTO is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * JEPETTO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details:
 * http://www.gnu.org/licenses/gpl.html
 */

package org.cytoscape.jepetto.internal;

import java.awt.BorderLayout;
import java.awt.Component;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import org.cytoscape.application.swing.CySwingApplication;
import org.cytoscape.application.swing.CytoPanel;
import org.cytoscape.application.swing.CytoPanelComponent;
import org.cytoscape.application.swing.CytoPanelName;
import org.cytoscape.application.swing.CytoPanelState;

public class JEPETTOResultsPanel extends JPanel implements CytoPanelComponent {

	/*******************
	 *   ATTRIBUTES    *
	 *******************/

	// Serial version UID (auto-generated)
	private static final long serialVersionUID = 8744772968653659654L;

	// Constructor attributes
	protected JPanel main;
	protected CySwingApplication desktopApp;
	private CytoPanel cytoPanelEast;
	private JTabbedPane resultsTabbedPanel;
	private String title;


	/*******************
	 *   CONSTRUCTOR   *
	 *******************/

	/**
	 * Default constructor
	 * creates an empty results panel
	 * @param desktopApp
	 */
	public JEPETTOResultsPanel(CySwingApplication desktopApp, String title){
		// Cytoscape properties
		this.desktopApp = desktopApp;
		this.title = title;
		this.cytoPanelEast = this.desktopApp.getCytoPanel(getCytoPanelName());

		// Basic tabbed panel
		this.setVisible(true);
		this.setLayout(new BorderLayout());
		this.resultsTabbedPanel = new JTabbedPane();

		this.add(resultsTabbedPanel, BorderLayout.CENTER);
	}



	/*******************
	 *     METHODS     *
	 *******************/

	/**
	 * Select the results panel
	 */
	protected void getPanel() {
		cytoPanelEast.setState(CytoPanelState.DOCK);
		int index;
		if (getTitle().equals(" Enrichment ")) {
			index = cytoPanelEast.indexOfComponent(CyActivator.enrichmentResultsTab);
		} else {
			index = cytoPanelEast.indexOfComponent(CyActivator.topologyResultsTab);
		}
		if (index == -1) {
			return;
		}
		cytoPanelEast.setSelectedIndex(index);
	}

	/**
	 * Get the results panel component
	 */
	public Component getComponent() {
		return this;
	}

	/**
	 * Get the results panel name
	 */
	public CytoPanelName getCytoPanelName() {
		return CytoPanelName.EAST;
	}

	/**
	 * Get the results panel title
	 */
	public String getTitle() {
		return this.title;
	}

	/**
	 * Get the results panel icon
	 */
	public Icon getIcon() {
		Icon icon = new ImageIcon(getClass().getResource("/logo_small.png"));
		return icon;
	}

	public JTabbedPane getResultsTabbedPanel() {
		return resultsTabbedPanel;
	}
}
