/*
 * Copyright (C) 2013-2014  Charles Winterhalter
 * School of Computing Science, Newcastle University
 *
 * This file is part of the Java Enrichment of Pathways Extended To Topology
 * (JEPETTO) plugin for Cytoscape.
 *
 * JEPETTO is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * JEPETTO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details:
 * http://www.gnu.org/licenses/gpl.html
 */

package org.cytoscape.jepetto.internal;

import java.awt.BorderLayout;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JViewport;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class JEPETTOSaveTable {

	/*******************
	 *    ATTRIBUTES   *
	 *******************/

	@SuppressWarnings("unused")
	private final Logger logger = LoggerFactory.getLogger(getClass());

	private final String tableName;
	private final JPanel tablePanel;
	private final JTable table;


	/*******************
	 *   CONSTRUCTOR   *
	 *******************/

	public JEPETTOSaveTable (String tableName, JPanel tablePanel, JTable table) {

		this.tableName = tableName;
		this.tablePanel = tablePanel;
		this.table = table;

		String filePath = displayChooser();

		JTable tableToSave = getTableToSave();

		if (filePath != null) {
			saveTable(filePath, tableToSave);
		}

	}


	/*******************
	 *     METHODS     *
	 *******************/

	/**
	 * Displays a panel to select the destination file
	 * @return path to file
	 */
	private String displayChooser() {
		String path = null;
		JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
		int savingValue = chooser.showSaveDialog(CyActivator.topologyResultsTab.getResultsTabbedPanel());
		if (savingValue == JFileChooser.APPROVE_OPTION) {
			try {
				path = chooser.getSelectedFile().getAbsolutePath();
				if (!path.endsWith(".txt") & !path.endsWith(".TXT")) {
					path += ".txt";
				}
				String fileNoExtension = path.substring(0, path.length()-4);
				if (new File(path).exists()) {
					int index = 1;
					path = fileNoExtension+"_"+index+".txt";
					while (new File(path).exists()) {
						index++;
						path = fileNoExtension+"_"+index+".txt";
					}
				}
			} catch (Exception e1) {
				e1.printStackTrace();
				return null;
			}
		}
		return path;
	}


	/**
	 * Find the reference to the correct table to save
	 * @return the table to save
	 */
	private JTable getTableToSave() {
		JTable table = null;
		if (tableName.equals("Ranking")) {
			table = this.table;
		} else if (tableName.equals("Statistics")) {
			BorderLayout bl = (BorderLayout) tablePanel.getLayout();
			table = (JTable) bl.getLayoutComponent(BorderLayout.CENTER);
		} else {
			BorderLayout bl = (BorderLayout) tablePanel.getLayout();
			JScrollPane scroll = (JScrollPane) bl.getLayoutComponent(BorderLayout.CENTER);
			JViewport viewport = scroll.getViewport();
			table = (JTable) viewport.getView();
		}
		return table;
	}


	/**
	 * Save a table content
	 * @param filePath
	 */
	private void saveTable(String filePath, JTable table) {
		try {
			BufferedWriter bfw = new BufferedWriter(new FileWriter(filePath));
			for(int i = 0 ; i < table.getColumnCount() ; i++) {
				bfw.write(table.getColumnName(i));
				if (i != table.getColumnCount()-1) {
					bfw.write("\t");
				}
			}
			bfw.newLine();
			for (int i = 0 ; i < table.getRowCount(); i++) {
				for(int j = 0 ; j < table.getColumnCount();j++) {
					bfw.write((String)(table.getValueAt(i,j)));
					if (j != table.getColumnCount()-1) {
						bfw.write("\t");
					}
				}
				bfw.newLine();
			}
			bfw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

}
