/*
 * Copyright (C) 2013-2014  Charles Winterhalter
 * School of Computing Science, Newcastle University
 *
 * This file is part of the Java Enrichment of Pathways Extended To Topology
 * (JEPETTO) plugin for Cytoscape.
 *
 * JEPETTO is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * JEPETTO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details:
 * http://www.gnu.org/licenses/gpl.html
 */

package org.cytoscape.jepetto.internal;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;

import org.cytoscape.application.swing.CytoPanelComponent;
import org.cytoscape.application.swing.CytoPanelName;


public class JEPETTOServerPanel extends JPanel implements CytoPanelComponent, ActionListener {

	/*******************
	 *   ATTRIBUTES    *
	 *******************/

	// Serial version UID (auto-generated)
	private static final long serialVersionUID = 8292806967891823933L;

	// Method choice
	protected static final JRadioButton enrichmentRadio = new JRadioButton("Enrichment");
	protected static final JRadioButton topologyRadio = new JRadioButton("Topology");

	// Analysis panels
	private JEPETTOEnrichNetPanel enrichmentPanel = null;
	private JEPETTOTopoGSAPanel topologyPanel = null;

	/*******************
	 *   CONSTRUCTOR   *
	 *******************/

	/**
	 * Initialize Cytoscape control panel
	 */
	public JEPETTOServerPanel() {

		// top panel with radio buttons
		JPanel topPanel = new JPanel();
		topPanel.setLayout(new GridLayout(1,2));
		topPanel.add(enrichmentRadio);
		topPanel.add(topologyRadio);

		enrichmentRadio.setSelected(true);

		// border for top panel
		Border border = BorderFactory.createLineBorder(Color.black);
		String title = "Please choose a method of analysis:";
		TitledBorder titledBorder = BorderFactory.createTitledBorder(border, title);
		titledBorder.setTitleJustification(TitledBorder.CENTER);
		topPanel.setBorder(titledBorder);

		// group radio buttons
		ButtonGroup group = new ButtonGroup();
		group.add(enrichmentRadio);
		group.add(topologyRadio);

		// set on click action
		enrichmentRadio.setToolTipText("Enrichment analysis of the input gene set");
		enrichmentRadio.addActionListener(this);

		topologyRadio.setToolTipText("Topological analysis of the input gene set");
		topologyRadio.addActionListener(this);

		// analysis panels
		enrichmentPanel = new JEPETTOEnrichNetPanel();
		topologyPanel = new JEPETTOTopoGSAPanel();

		// extra space at the bottom
		JPanel bottomPanel = new JPanel();

		// dimensions
		int width = (int)(0.25 * Toolkit.getDefaultToolkit().getScreenSize().getWidth());
		this.setPreferredSize(new Dimension(width, 0));

		// layout
		this.setLayout(new BorderLayout(2,16));
		this.add(topPanel, BorderLayout.NORTH);
		this.add(enrichmentPanel, BorderLayout.CENTER);
		this.add(bottomPanel, BorderLayout.SOUTH);
	}



	/*******************
	 *     METHODS     *
	 *******************/

	/**
	 * Get Cytoscape control panel
	 */
	@Override
	public Component getComponent() {
		return this;
	}

	/**
	 * Get panel name
	 */
	@Override
	public CytoPanelName getCytoPanelName() {
		return CytoPanelName.WEST;
	}

	/**
	 * Get panel title: logo
	 */
	@Override
	public String getTitle() {
		return "";
	}

	/**
	 * Get panel icon
	 */
	@Override
	public Icon getIcon() {
		Icon icon = new ImageIcon(getClass().getResource("/logo_small.png"));
		return icon;
	}

	/**
	 * Listen for user action.
	 * Initialize enrichment or topological analysis panel.
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == topologyRadio) {
			this.remove(enrichmentPanel);
			this.add(topologyPanel, BorderLayout.CENTER);
			this.repaint();
		} else if (e.getSource() == enrichmentRadio) {
			this.remove(topologyPanel);
			this.add(enrichmentPanel, BorderLayout.CENTER);
			this.repaint();
		}
	}
}
